from __future__ import annotations

from collections import defaultdict
from datetime import datetime
from types import FrameType, TracebackType
from typing import TYPE_CHECKING, Callable, Literal, Self, TypeAlias

if TYPE_CHECKING:
    from logbook.handlers import Handler

from _typeshed import Incomplete

StringLevel: TypeAlias = Literal['CRITICAL', 'ERROR', 'WARNING', 'INFO', 'DEBUG', 'NOTICE', 'TRACE', 'NOTSET']
# Need to keep in sync with the constansts from logbook.base
NumericLevel: TypeAlias = Literal[15, 14, 13, 12, 11, 10, 9, 0]
LogLevel: TypeAlias = StringLevel | NumericLevel

# Type for log filter functions
LogFilter: TypeAlias = Callable[[LogRecord, Handler], bool]

# Level constants
CRITICAL: Literal[15] = 15
ERROR: Literal[14] = 14
WARNING: Literal[13] = 13
INFO: Literal[12] = 12
DEBUG: Literal[11] = 11
NOTICE: Literal[10] = 10
TRACE: Literal[9] = 9
NOTSET: Literal[0] = 0

_ExcInfo = tuple[type[BaseException], BaseException, TracebackType] | tuple[None, None, None]

class LogRecord:
    channel: str
    msg: str
    args: tuple[Incomplete, ...]
    kwargs: dict[str, Incomplete]
    level: LogLevel
    exc_info: _ExcInfo | Literal[True] | None
    extra: defaultdict[str, Incomplete]
    frame: FrameType | None
    frame_correction: int
    process: int | None
    time: datetime | None
    keep_open: bool
    heavy_initialized: bool
    late: bool
    information_pulled: bool

    def __init__(
        self,
        channel: str,
        level: int | StringLevel,
        msg: str,
        args: tuple[Incomplete, ...] | None = ...,
        kwargs: dict[str, Incomplete] | None = ...,
        exc_info: _ExcInfo | Literal[True] | BaseException | None = ...,
        extra: dict[str, Incomplete] | None = ...,
        frame: FrameType | None = ...,
        dispatcher: Incomplete = ...,
        frame_correction: int = ...,
    ) -> None: ...
    @property
    def message(self) -> str: ...
    @property
    def level_name(self) -> str: ...
    @property
    def calling_frame(self) -> FrameType | None: ...
    @property
    def func_name(self) -> str | None: ...
    @property
    def module(self) -> str | None: ...
    @property
    def filename(self) -> str | None: ...
    @property
    def lineno(self) -> int | None: ...
    @property
    def greenlet(self) -> int | None: ...
    @property
    def thread(self) -> int | None: ...
    @property
    def thread_name(self) -> str | None: ...
    @property
    def process_name(self) -> str | None: ...
    @property
    def formatted_exception(self) -> str | None: ...
    @property
    def exception_name(self) -> str | None: ...
    @property
    def exception_shortname(self) -> str | None: ...
    @property
    def exception_message(self) -> str | None: ...
    @property
    def dispatcher(self) -> Incomplete: ...
    def heavy_init(self) -> None: ...
    def pull_information(self) -> None: ...
    def close(self) -> None: ...
    def to_dict(self, json_safe: bool = ...) -> dict[str, Incomplete]: ...
    @classmethod
    def from_dict(cls, d: dict[str, Incomplete]) -> LogRecord: ...
    def update_from_dict(self, d: dict[str, Incomplete]) -> LogRecord: ...

class RecordDispatcher:
    """A record dispatcher is the internal base class that implements
    the logic used by the :class:`~logbook.Logger`.
    """

    suppress_dispatcher: bool
    name: str | None
    handlers: list['Handler']
    group: LoggerGroup | None
    level: int
    disabled: bool

    def __init__(self, name: str | None = None, level: int = ...) -> None: ...
    def handle(self, record: LogRecord) -> None: ...
    def make_record_and_handle(
        self,
        level: int,
        msg: str,
        args: tuple[Incomplete, ...],
        kwargs: dict[str, Incomplete],
        exc_info: _ExcInfo | Literal[True] | BaseException | None,
        extra: dict[str, Incomplete] | None,
        frame_correction: int,
    ) -> None: ...
    def call_handlers(self, record: LogRecord) -> None: ...

class LoggerMixin:
    """This mixin class defines and implements the "usual" logger
    interface (i.e. the descriptive logging functions).
    """

    level_name: str

    def trace(self, *args: Incomplete, **kwargs: Incomplete) -> None: ...
    def debug(self, *args: Incomplete, **kwargs: Incomplete) -> None: ...
    def info(self, *args: Incomplete, **kwargs: Incomplete) -> None: ...
    def notice(self, *args: Incomplete, **kwargs: Incomplete) -> None: ...
    def warning(self, *args: Incomplete, **kwargs: Incomplete) -> None: ...
    def error(self, *args: Incomplete, **kwargs: Incomplete) -> None: ...
    def critical(self, *args: Incomplete, **kwargs: Incomplete) -> None: ...
    def exception(self, *args: Incomplete, **kwargs: Incomplete) -> None: ...
    def log(self, level: int, *args: Incomplete, **kwargs: Incomplete) -> None: ...

class Logger(RecordDispatcher, LoggerMixin):
    """Instances of the Logger class represent a single logging channel."""

    def __init__(self, name: str | None = None, level: int = ...) -> None: ...

class LoggerGroup:
    """A LoggerGroup represents a group of loggers."""

    loggers: list[Logger]
    level: int
    disabled: bool
    processor: Incomplete | None

    def __init__(
        self,
        loggers: list[Logger] | None = None,
        level: int = ...,
        processor: Incomplete | None = None,
    ) -> None: ...
    def add_logger(self, logger: Logger) -> None: ...
    def remove_logger(self, logger: Logger) -> None: ...
    def process_record(self, record: LogRecord) -> None: ...

class Processor:
    """A context manager that processes log records with a given function."""

    def __init__(self, callback: Callable[[LogRecord], None]) -> None: ...
    def __enter__(self) -> Processor: ...
    def __exit__(self, exc_type: object, exc_value: object, tb: object) -> None: ...

class ContextObject:
    """An object that can be bound to a context."""

    stack_manager: Incomplete | None

    def push_context(self) -> None: ...
    def pop_context(self) -> None: ...
    def push_application(self) -> None: ...
    def pop_application(self) -> None: ...
    def __enter__(self) -> Self: ...
    def __exit__(self, exc_type: object, exc_value: object, tb: object) -> None: ...

class Flags:
    """Allows flags to be pushed on a flag stack."""

    stack_manager: Incomplete

    def __init__(self, **flags: Incomplete) -> None: ...
    @staticmethod
    def get_flag(flag: str, default: Incomplete = ...) -> Incomplete: ...
    def push_application(self) -> None: ...
    def pop_application(self) -> None: ...
    def push_context(self) -> None: ...
    def pop_context(self) -> None: ...
    def __enter__(self) -> Flags: ...
    def __exit__(self, exc_type: object, exc_value: object, tb: object) -> None: ...

__all__ = [
    'CRITICAL',
    'DEBUG',
    'ERROR',
    'INFO',
    'NOTICE',
    'NOTSET',
    'TRACE',
    'WARNING',
    'StringLevel',
    'NumericLevel',
    'LogLevel',
    'LogFilter',
    'LogRecord',
    'Logger',
    'LoggerGroup',
    'Processor',
    'ContextObject',
    'Flags',
]



# Configuration file for the Sphinx documentation builder.
#
# For the full list of built-in configuration values, see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

# -- Project information -----------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#project-information

project = 'ChameleonLog'
copyright = '2025, Nguyễn Hồng Quân'
author = 'Nguyễn Hồng Quân <ng.hong.quan@gmail.com>'

# -- General configuration ---------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#general-configuration

extensions = [
    'sphinx.ext.autodoc',
    'sphinx.ext.intersphinx',
    'myst_parser',
    'sphinxcontrib.asciinema',
]

templates_path = ['_templates']
exclude_patterns = ['_build', 'Thumbs.db', '.DS_Store']


# -- Options for HTML output -------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#options-for-html-output

html_theme = 'sphinx_book_theme'
html_static_path = ['_static']

# -- Intersphinx configuration -----------------------------------------------
intersphinx_mapping = {
    'python': ('https://docs.python.org/3', None),
    'logbook': ('https://logbook.readthedocs.io/en/stable/', None),
    'rich': ('https://rich.readthedocs.io/en/stable/', None),
}

# -- Autodoc configuration ---------------------------------------------------
autodoc_default_options = {
    'members': True,
    'undoc-members': True,
    'show-inheritance': True,
}



from __future__ import annotations

from typing import TYPE_CHECKING

import logbook
import pytest


if TYPE_CHECKING:
    from logbook.base import Logger


@pytest.fixture
def logger() -> Logger:
    return logbook.Logger('testlogger')



#!/usr/bin/env python3
"""Simple example of using JournaldHandler with Logbook."""

import getpass
import platform

import logbook
from logbook import Processor

from chameleon_log.journald import JournaldHandler


def main() -> None:
    """Main function demonstrating JournaldHandler usage."""
    # Create a JournaldHandler
    handler = JournaldHandler(syslog_identifier='example-simple')

    with handler:
        # Get a logger
        logger = logbook.Logger(__name__)

        # Log messages at different levels
        logger.debug('This is a debug message')
        logger.info('Application started successfully')
        logger.notice('This is a notice message')
        logger.warning('This is a warning message')
        logger.error('An error occurred')

        # Example 1: Pass extra fields directly using extra= parameter
        user = getpass.getuser()
        logger.info(
            'Current Linux user: {}',
            user,
            extra={'linux': platform.freedesktop_os_release(), 'platform': platform.platform()},
        )

        # Example 2: Cause an error by calling int('abc') and log the exception
        try:
            # This will raise a ValueError
            int('abc')
        except ValueError:
            logger.exception('Failed to convert string to integer')

        # Example 3: Use a Processor to inject context into multiple log calls
        def inject_error_context(record: logbook.LogRecord) -> None:
            """Inject error context into log records."""
            record.extra['error_type'] = 'conversion'

        def get_optional_message() -> None:
            """Return no message to demonstrate a None-related failure."""
            return None

        # Log an exception with extra fields using a Processor
        try:
            # Another error example
            get_optional_message().strip()  # type: ignore
        except AttributeError:
            with Processor(inject_error_context):
                logger.exception('An error occurred during processing')


if __name__ == '__main__':
    main()



from __future__ import annotations

from logbook.base import (
    CRITICAL as CRITICAL,
    DEBUG as DEBUG,
    ERROR as ERROR,
    INFO as INFO,
    NOTICE as NOTICE,
    NOTSET as NOTSET,
    TRACE as TRACE,
    WARNING as WARNING,
    Logger as Logger,
    LoggerGroup as LoggerGroup,
    LogLevel as LogLevel,
    LogRecord as LogRecord,
    Processor as Processor,
)
